
// Projekt: CDWserver - CDs brennen im Netz
// Datei: Process.cpp
// Autor: Jrg Haeger, 02.11.1999

#include <string.h>
#include <sys/wait.h>
#include <unistd.h>

#include "Support.h"
#include "Log.h"
#include "Process.h"

Process::Process(const char *fileName, const char *outName)
{
	argsNum = envsNum = 0;
	args[argsNum++] = strDup(fileName);
	args[argsNum] = NULL;
	envs[envsNum] = NULL;
	pid = -1;
	inStream = NULL;
	if (outName == NULL)
		outStream = NULL;
	else
		outStream = fopen(outName, "w");
	linePos = 0;
	lineStr[0] = 0;
}

Process::~Process()
{
	log.putv("Process::~Process() %s\n", args[0]);
	if (outStream != NULL)
		fclose(outStream);
	if (inStream != NULL)
		fclose(inStream);
	for (int i = 0; i < envsNum; i++)
		delete envs[i];
	for (int i = 0; i < argsNum; i++)
		delete args[i];
}

void Process::addArg(const char *argStr)
{
	args[argsNum++] = strDup(argStr);
	args[argsNum] = NULL;
}

int Process::getResult()
{
	fclose(inStream);
	inStream = NULL;
	int status;
	waitpid(pid, &status, 0);
	if (WIFEXITED(status))
		return WEXITSTATUS(status);
	else
		return -1;
}

const char *Process::readLine()
{
	while (1)
	{
		int ch = fgetc(inStream);
		if (ch == 8)
		{
			if (linePos > 0)
				linePos--;
		}
		else
		if (ch == '\r')
			linePos = 0;
		else
		if (ch == '\n')
		{
			linePos = 0;
			lineStr[0] = 0;
		}
		else
		{
			lineStr[linePos++] = ch;
			break;
		}
	}

	while (linePos < sizeof lineStr - 2)
	{
		int ch = fgetc(inStream);
		if (ch == EOF)
			return NULL;
		if (ch == 8 || ch == '\r' || ch == '\n')
		{
			ungetc(ch, inStream);
			break;
		}
		lineStr[linePos++] = ch;
	}
	lineStr[linePos] = '\n';;
	lineStr[linePos+1] = 0;
	if (outStream != NULL)
		fputs(lineStr, outStream);
	return lineStr;
}

void Process::show()
{
	log.putv("Process: <%s>\n", args[0]);
	for (int i = 1; i < argsNum; i++)
		log.putv("%8d: <%s>\n", i, args[i]);
}

/**
	@see glibc-2.1.1/sysdeps/posix/pipestream.c
*/
void Process::start(const char *workDirName)
{
	int pipedes[2];
	if (pipe(pipedes) != 0)
		throw "pipe failed";

	pid = fork();
	if (pid == (pid_t)-1)
	{
		// The fork failed.
		close(pipedes[0]);
		close(pipedes[1]);
		throw "fork failed";
	}

	if (pid == (pid_t)0)
	{
		// We are the child side.
		dup2(pipedes[STDOUT_FILENO], STDOUT_FILENO);
		dup2(pipedes[STDOUT_FILENO], STDERR_FILENO);

		// Close the pipe descriptors.
		close(pipedes[STDIN_FILENO]);
		close(pipedes[STDOUT_FILENO]);

		if (workDirName != NULL)
			if (chdir(workDirName) != 0)
				exit(1);

//		execve(args[0], args, envs);
		execvp(args[0], args);
		exit(1);
	}

	close(pipedes[STDOUT_FILENO]);
	inStream = fdopen(pipedes[STDIN_FILENO], "r");
	if (inStream == NULL)
		throw "open inStream failed";
}

const char *Process::toString(char *str)
{
	sprintf(str, "Process: <%s", args[0]);
	for (int i = 1; i < argsNum; i++)
	{
		strcat(str, " ");
		strcat(str, args[i]);
	}
	strcat(str, ">");
	return str;
}

